//
// Institute for Signal Processing (University of Luebeck, Germany)
// Copyright (c) 2011 by Radoslaw Mazur
//
// Permission to use, copy, modify, and distribute this software without
// fee is hereby granted FOR RESEARCH/EDUCATION PURPOSES only, provided
// that this copyright notice appears in all copies and in all supporting
// documentation, and that the software is not redistributed for any
// fee (except for a nominal shipping charge).
//
// For any other uses of this software, in original or modified form,
// including but not limited to consulting, production or distribution
// in whole or in part, specific prior permission must be obtained
// from the author.
// Signal processing methods and algorithms implemented by this
// software may be claimed by patents owned by others.
//
// The author makes no representation about the suitability of this
// software for any purpose. It is provided "as is" without warranty
// of any kind, either expressed or implied.
// Beware of the bugs.
//
//     Revision history
//
//     Ver     Date         Description
//     -------------------------------------------------------------------
//     0.5     21-08-2011   basic version


#include "mex.h"
#include <iostream>
#include <stdio.h>
#include <math.h>
#include <time.h>

using namespace std;

void do_the_ica_2d(double *re1,double *im1,double *re2,double *im2,
		double *ret_re, double *ret_im, int iterations, float stepsize, int fbins, int dpoints);



void mexFunction( int nlhs, mxArray *plhs[],
                  int nrhs, const mxArray *prhs[] ){


  printf("Time-Frequency ICA using CUDA\n");

  /* Check for proper number of arguments. */
  if(nrhs!=4) {
    mexErrMsgTxt("Four inputs required.");
  } else if(nlhs>1) {
    mexErrMsgTxt("Too many output arguments.");
  }


  int iterations = *mxGetPr(prhs[2]);
  float stepsize = *mxGetPr(prhs[3]);;


  int fbins    = mxGetM(prhs[0]);
  int dpoints  = mxGetN(prhs[0]);
  int fbins2   = mxGetM(prhs[1]);
  int dpoints2 = mxGetN(prhs[1]);

  /* The input must be a noncomplex scalar double.*/
  if( !mxIsDouble(prhs[0]) || !mxIsComplex(prhs[0])) {
    mexErrMsgTxt("Input 1 must be a complex array.");
  }

  if( !mxIsDouble(prhs[1]) || !mxIsComplex(prhs[1])) {
    mexErrMsgTxt("Input 2 must be a complex array.");
  }

  if( (fbins!=fbins2) || (dpoints!=dpoints2)) {
    mexErrMsgTxt("Arrays are different size");
  }
  double *re1 = mxGetPr(prhs[0]);
  double *im1 = mxGetPi(prhs[0]);
  double *re2 = mxGetPr(prhs[1]);
  double *im2 = mxGetPi(prhs[1]);


  /* Create matrix for the return argument. */
  plhs[0] = mxCreateDoubleMatrix(4,fbins, mxCOMPLEX);
  double *ret_re = mxGetPr(plhs[0]);
  double *ret_im = mxGetPi(plhs[0]);


  do_the_ica_2d(re1, im1, re2, im2, ret_re, ret_im, iterations, stepsize, fbins, dpoints);

}

