//
// Institute for Signal Processing (University of Luebeck, Germany)
// Copyright (c) 2011 by Radoslaw Mazur
//
// Permission to use, copy, modify, and distribute this software without
// fee is hereby granted FOR RESEARCH/EDUCATION PURPOSES only, provided
// that this copyright notice appears in all copies and in all supporting
// documentation, and that the software is not redistributed for any
// fee (except for a nominal shipping charge).
//
// For any other uses of this software, in original or modified form,
// including but not limited to consulting, production or distribution
// in whole or in part, specific prior permission must be obtained
// from the author.
// Signal processing methods and algorithms implemented by this
// software may be claimed by patents owned by others.
//
// The author makes no representation about the suitability of this
// software for any purpose. It is provided "as is" without warranty
// of any kind, either expressed or implied.
// Beware of the bugs.
//
//     Revision history
//
//     Ver     Date         Description
//     -------------------------------------------------------------------
//     0.5     21-08-2011   basic version


#include <iostream>
#include <stdio.h>
#include <math.h>
#include <time.h>


using namespace std;

#include "../include/icaData.h"

__global__ void par_ica_2d(icaData *d);
__global__ void par_ica_3d(icaData_3d *d);

void do_the_ica_2d(double *re1,double *im1,double *re2,double *im2,
		double *ret_re, double *ret_im, int iterations, float stepsize, int fbins, int dpoints) {

            //clock_t start, start_ges, end;
            //start_ges = clock();

            // Generate object with all informations
            icaData d(dpoints,fbins,iterations);
            int data_ges = dpoints * fbins;

            // Change format double --> float
            // Change interleaving:  Matlab independent re,im <--> interleaved
            for (int i=0;i<data_ges;i++) {
                d.in[i] = cu2dVect(cuComplex(re1[i], im1[i]),cuComplex(re2[i],im2[i]));
            }

            // Init unmixing matrices
            for (int i=0;i<fbins;i++) {
                d.w[i] = cu2dMatrix(1,0,0,1);
            }

            // Copy d.in and d.w to device memory
            d.copyToDevice();
            icaData *dev_d;

            // Copy d itself to device memory, for easy access of all data structures from the kernel
            cudaMalloc( (void**)&dev_d, sizeof(icaData));
            cudaMemcpy(dev_d, &d, sizeof(icaData), cudaMemcpyHostToDevice);

            //start = clock();

            // The actual computation
            par_ica_2d<<<128,64>>>(dev_d);

            //cudaThreadSynchronize();
            //end = clock();

            // Copy result to host
            d.copyToHost();


            //cout << "Total time:  " << ((double)(end-start))/((double)(CLOCKS_PER_SEC)) << "s" <<endl;
            //cout << "Kernel time: " << ((double)(end-start_ges))/((double)(CLOCKS_PER_SEC)) << "s" <<endl;

            // Retrieve result:
            // Change format float --> double
            // Change interleaving:  interleaved <--> Matlab independent re,im
            for (int i=0;i<fbins;i++) {
                ret_re[i*4] = d.w[i].value11.r;
                ret_im[i*4] = d.w[i].value11.i;
                ret_re[i*4+1] = d.w[i].value12.r;
                ret_im[i*4+1] = d.w[i].value12.i;
                ret_re[i*4+2] = d.w[i].value21.r;
                ret_im[i*4+2] = d.w[i].value21.i;
                ret_re[i*4+3] = d.w[i].value22.r;
                ret_im[i*4+3] = d.w[i].value22.i;
            }
            cudaFree(dev_d);
		}



void do_the_ica_3d(double *re1,double *im1,double *re2,double *im2,double *re3,double *im3,
		double *ret_re, double *ret_im, int iterations, float stepsize, int fbins, int dpoints) {

            //clock_t start, start_ges, end;
            //start_ges = clock();

            icaData_3d d(dpoints,fbins,iterations);
            int data_ges = dpoints * fbins;


            for (int i=0;i<data_ges;i++) {
                d.in[i] = cu3dVect(cuComplex(re1[i], im1[i]),cuComplex(re2[i],im2[i]),cuComplex(re3[i],im3[i]));
            }

            for (int i=0;i<fbins;i++) {
                d.w[i] = cu3dMatrix(1,0,0,  0,1,0,   0,0,1);
            }


            d.copyToDevice();
            icaData_3d *dev_d;

            cudaMalloc( (void**)&dev_d, sizeof(icaData_3d));
            cudaMemcpy(dev_d, &d, sizeof(icaData_3d), cudaMemcpyHostToDevice);



            //start = clock();

            par_ica_3d<<<128,64>>>(dev_d);
            //par_m_mult_var<<<128,64>>>(dev_d);
            //cudaThreadSynchronize();
            //end = clock();
            d.copyToHost();
            //cout << "Total time:  " << ((double)(end-start))/((double)(CLOCKS_PER_SEC)) << "s" <<endl;
            //cout << "Kernel time: " << ((double)(end-start_ges))/((double)(CLOCKS_PER_SEC)) << "s" <<endl;


            for (int i=0;i<fbins;i++) {
                ret_re[i*9] = d.w[i].value11.r;
                ret_im[i*9] = d.w[i].value11.i;

                ret_re[i*9+1] = d.w[i].value12.r;
                ret_im[i*9+1] = d.w[i].value12.i;

                ret_re[i*9+2] = d.w[i].value13.r;
                ret_im[i*9+2] = d.w[i].value13.i;


                ret_re[i*9+3] = d.w[i].value21.r;
                ret_im[i*9+3] = d.w[i].value21.i;

                ret_re[i*9+4] = d.w[i].value22.r;
                ret_im[i*9+4] = d.w[i].value22.i;

                ret_re[i*9+5] = d.w[i].value23.r;
                ret_im[i*9+5] = d.w[i].value23.i;


                ret_re[i*9+6] = d.w[i].value31.r;
                ret_im[i*9+6] = d.w[i].value31.i;

                ret_re[i*9+7] = d.w[i].value32.r;
                ret_im[i*9+7] = d.w[i].value32.i;

                ret_re[i*9+8] = d.w[i].value33.r;
                ret_im[i*9+8] = d.w[i].value33.i;
            }
            cudaFree(dev_d);
		}
