//
// Institute for Signal Processing (University of Luebeck, Germany)
// Copyright (c) 2011 by Radoslaw Mazur
//
// Permission to use, copy, modify, and distribute this software without
// fee is hereby granted FOR RESEARCH/EDUCATION PURPOSES only, provided
// that this copyright notice appears in all copies and in all supporting
// documentation, and that the software is not redistributed for any
// fee (except for a nominal shipping charge).
//
// For any other uses of this software, in original or modified form,
// including but not limited to consulting, production or distribution
// in whole or in part, specific prior permission must be obtained
// from the author.
// Signal processing methods and algorithms implemented by this
// software may be claimed by patents owned by others.
//
// The author makes no representation about the suitability of this
// software for any purpose. It is provided "as is" without warranty
// of any kind, either expressed or implied.
// Beware of the bugs.
//
//     Revision history
//
//     Ver     Date         Description
//     -------------------------------------------------------------------
//     0.5     21-08-2011   basic version



#include "../include/icaData.h"



icaData::icaData(): data_points(0), f_bins(1), iterations(200) {};
icaData::icaData(int d_points, int fb): data_points(d_points), f_bins(fb), data_ges(d_points*fb), iterations(200), step_size(0.02f) {
    int DATA_ges = data_points * f_bins;
    in  = new cu2dVect[data_ges];
    out = new cu2dVect[data_ges];
    w = new cu2dMatrix[f_bins];

    cudaMalloc( (void**)&dev_in,  DATA_ges   * sizeof(cu2dVect));
    cudaMalloc( (void**)&dev_out, DATA_ges   * sizeof(cu2dVect));
    cudaMalloc( (void**)&dev_w,   f_bins * sizeof(cu2dMatrix));
};

icaData::icaData(int d_points, int fb, int it): data_points(d_points), f_bins(fb), data_ges(d_points*fb), iterations(it), step_size(0.02f) {
    int DATA_ges = data_points * f_bins;
    in  = new cu2dVect[data_ges];
    out = new cu2dVect[data_ges];
    w = new cu2dMatrix[f_bins];

    cudaMalloc( (void**)&dev_in,  DATA_ges   * sizeof(cu2dVect));
    cudaMalloc( (void**)&dev_out, DATA_ges   * sizeof(cu2dVect));
    cudaMalloc( (void**)&dev_w,   f_bins * sizeof(cu2dMatrix));
};


icaData::~icaData(){
    delete in;
    delete out;
    delete w;
    cudaFree(dev_in);
    cudaFree(dev_out);
    cudaFree(dev_w);
};

void icaData::copyToDevice(){
    cudaMemcpy(dev_in, in, data_ges   * sizeof(cu2dVect), cudaMemcpyHostToDevice);
    cudaMemcpy(dev_w, w,   f_bins * sizeof(cu2dMatrix), cudaMemcpyHostToDevice);
};
void icaData::copyToHost(){
    //cudaMemcpy(out, dev_out, data_ges   * sizeof(cu2dVect), cudaMemcpyDeviceToHost);
    cudaMemcpy(w,  dev_w,    f_bins * sizeof(cu2dMatrix), cudaMemcpyDeviceToHost);
};

















icaData_3d::icaData_3d(): data_points(0), f_bins(1), iterations(200) {};
icaData_3d::icaData_3d(int d_points, int fb): data_points(d_points), f_bins(fb), data_ges(d_points*fb), iterations(200), step_size(0.02f) {
    int DATA_ges = data_points * f_bins;
    inv_mlen = (1.0f/(float)data_points);
    in  = new cu3dVect[data_ges];
    out = new cu3dVect[data_ges];
    w = new cu3dMatrix[f_bins];

    cudaMalloc( (void**)&dev_in,  DATA_ges   * sizeof(cu3dVect));
    cudaMalloc( (void**)&dev_out, DATA_ges   * sizeof(cu3dVect));
    cudaMalloc( (void**)&dev_w,   f_bins * sizeof(cu3dMatrix));
};

icaData_3d::icaData_3d(int d_points, int fb, int it): data_points(d_points), f_bins(fb), data_ges(d_points*fb), iterations(it), step_size(0.02f) {
    int DATA_ges = data_points * f_bins;
    inv_mlen = (1.0f/(float)data_points);
    in  = new cu3dVect[data_ges];
    out = new cu3dVect[data_ges];
    w = new cu3dMatrix[f_bins];

    cudaMalloc( (void**)&dev_in,  DATA_ges   * sizeof(cu3dVect));
    cudaMalloc( (void**)&dev_out, DATA_ges   * sizeof(cu3dVect));
    cudaMalloc( (void**)&dev_w,   f_bins * sizeof(cu3dMatrix));
};


icaData_3d::~icaData_3d(){
    delete in;
    delete out;
    delete w;
    cudaFree(dev_in);
    cudaFree(dev_out);
    cudaFree(dev_w);
};

void icaData_3d::copyToDevice(){
    cudaMemcpy(dev_in, in, data_ges   * sizeof(cu3dVect), cudaMemcpyHostToDevice);
    cudaMemcpy(dev_w, w,   f_bins * sizeof(cu3dMatrix), cudaMemcpyHostToDevice);
};
void icaData_3d::copyToHost(){
    //cudaMemcpy(out, dev_out, data_ges   * sizeof(cu3dVect), cudaMemcpyDeviceToHost);
    cudaMemcpy(w,  dev_w,    f_bins * sizeof(cu3dMatrix), cudaMemcpyDeviceToHost);
};






