//
// Institute for Signal Processing (University of Luebeck, Germany)
// Copyright (c) 2011 by Radoslaw Mazur
//
// Permission to use, copy, modify, and distribute this software without
// fee is hereby granted FOR RESEARCH/EDUCATION PURPOSES only, provided
// that this copyright notice appears in all copies and in all supporting
// documentation, and that the software is not redistributed for any
// fee (except for a nominal shipping charge).
//
// For any other uses of this software, in original or modified form,
// including but not limited to consulting, production or distribution
// in whole or in part, specific prior permission must be obtained
// from the author.
// Signal processing methods and algorithms implemented by this
// software may be claimed by patents owned by others.
//
// The author makes no representation about the suitability of this
// software for any purpose. It is provided "as is" without warranty
// of any kind, either expressed or implied.
// Beware of the bugs.
//
//     Revision history
//
//     Ver     Date         Description
//     -------------------------------------------------------------------
//     0.5     21-08-2011   basic version


#include <iostream>
#include <stdio.h>


using namespace std;

#include "basic_data_types.cu"
#include "../include/icaData.h"

__device__ __host__ cu2dMatrix cu2dVect::kProd(const cu2dVect a) const {
  return cu2dMatrix(value1*a.value1,value2*a.value1,value1*a.value2,value2*a.value2);
}

__device__ __host__ cu3dMatrix cu3dVect::kProd(const cu3dVect a) const {
  return cu3dMatrix(value1*a.value1,value2*a.value1,value3*a.value1,
                    value1*a.value2,value2*a.value2,value3*a.value2,
                    value1*a.value3,value2*a.value3,value3*a.value3);
}

__global__ void par_ica_2d(icaData *d) {
    int tid = threadIdx.x +blockIdx.x * blockDim.x;
    if (tid< d->f_bins) {
      cu2dMatrix W = (d->dev_w)[tid];
      cuComplex    inv_mlen(1.0f/(float)d->data_points);
      for (int j=0;j<d->iterations;j++) {
        cu2dMatrix tmp;
        for (int i=0;i<d->data_points;i++) {
            cu2dVect y =  (W *  d->dev_in[tid + i* d->f_bins]);
            tmp += (y.nonLinearF()).kProd(y.conj());
        }
        cu2dMatrix II(1,0,0,1);
        W = (((II-(tmp * inv_mlen)) * W) * d-> step_size) +W ;
      }
    d->dev_w[tid] = W;
    }
}


void par_ica_2d_cpu(icaData *d) {
    //int tid = threadIdx.x +blockIdx.x * blockDim.x;
    for (int tid = 0; tid < d->f_bins;tid++) {
      cu2dMatrix W = (d->w)[tid];
      cuComplex    inv_mlen(1.0f/(float)d->data_points);
      //int offset = tid * d->data_points;
      for (int j=0;j<d->iterations;j++) {
        cu2dMatrix tmp;
        for (int i=0;i<d->data_points;i++) {
            //cu2dVect y =  (W *  d->in[tid + i* d->f_bins]);
            cu2dVect y =  (W *  d->in[tid * d->data_points + i]);
            tmp += (y.nonLinearF()).kProd(y.conj());
        }
        cu2dMatrix II(1,0,0,1);
        W = (((II-(tmp * inv_mlen)) * W) * d-> step_size) +W ;
      }
    d->w[tid] = W;
    }
}



__global__ void par_ica_3d(icaData_3d *d) {
    int tid = threadIdx.x +blockIdx.x * blockDim.x;
    if (tid< d->f_bins) {
      cu3dMatrix W = (d->dev_w)[tid];
      const cuComplex  inv_mlen(1.0f/(float)d->data_points);
      for (int j=0;j<d->iterations;j++) {
        cu3dMatrix tmp;
        for (int i=0;i<d->data_points;i++) {
            cu3dVect y =  (W *  d->dev_in[tid + i* d->f_bins]);
            tmp += (y.nonLinearF()).kProd(y.conj());
        }
        cu3dMatrix II(1,0,0,  0,1,0,  0,0,1);
        W = (((II-(tmp * inv_mlen)) * W) * d-> step_size) +W ;
      }
    d->dev_w[tid] = W;
    }
}

void par_ica_3d_cpu(icaData_3d *d) {
    for (int tid = 0; tid < d->f_bins;tid++) {
//        cout << tid << endl;
      cu3dMatrix W = (d->w)[tid];
      const cuComplex  inv_mlen(1.0f/(float)d->data_points);
      //int offset = tid * d->data_points;
      for (int j=0;j<d->iterations;j++) {
        cu3dMatrix tmp;
        for (int i=0;i<d->data_points;i++) {
            //cu3dVect y =  (W *  d->in[tid *  d->data_points + i]);
            cu3dVect y =  (W *  d->in[tid + i* d->f_bins]);
            tmp += (y.nonLinearF()).kProd(y.conj());
        }
        cu3dMatrix II(1,0,0,  0,1,0,  0,0,1);
        W = (((II-(tmp * inv_mlen)) * W) * d-> step_size) +W ;
      }
    d->w[tid] = W;
    }
}



